@@ -4,11 +4,11 @@ Next Reaction Method
 This module performs the Next Reaction Method from Gibson and Bruck [1]. Therefore, it is also called the Gibson and Bruck algorithm.
 
 [1] M.A. Gibson and J. "Bruck Efficient Exact Stochastic Simulation of Chemical Systems with Many Species and Many
-Channels", J. Phys. Chem., 2000,104,1876-1889
+Channels", J. Phys. Chem., 2000, 104, 1876-1889
 
-Written by TR Maarleveld, Amsterdam, The Netherlands
+Written by T.R. Maarleveld, Amsterdam, The Netherlands
 E-mail: tmd200@users.sourceforge.net
-Last Change: July 4, 2012
+Last Change: April 08, 2013
 """
 
 from stochpy import model_dir
@@ -70,35 +70,34 @@ class NextReactionMethod(PySCeS_Connector):
               self.ModelDir = model_dir
   
           self.N_matrix = copy.deepcopy(self.parse.N_matrix.transpose())   # June 5th 2012
-          self.X_matrixinit = copy.deepcopy(self.parse.X_matrix.transpose()[0])	 # 22/01/11   
+          self.X_matrixinit = copy.deepcopy(self.parse.X_matrix.transpose()[0])	 # January 21 2011
           self.reactants = copy.deepcopy(self.parse.reactants)
           self.species_stochmatrix = copy.deepcopy(self.parse.species)
           self.n_species = len(self.X_matrixinit)
           self.reagents = copy.deepcopy(self.parse.reagents)
           self.rate_names = copy.deepcopy(self.parse.reactions)
           self.propensities = copy.deepcopy(self.parse.propensities)
-          self.fixed_species= copy.deepcopy(self.parse.fixed_species)
+          self.fixed_species = copy.deepcopy(self.parse.fixed_species)
           self.fixed_species_amount = copy.deepcopy(self.parse.fixed_species_amount)
           self.affects = copy.deepcopy(self.parse.affects)
-          self.dep_graph = copy.deepcopy(self.parse.dep_graph)          
+          self.dep_graph = copy.deepcopy(self.parse.dep_graph)    
           self.aDict = copy.deepcopy(self.parse.Mod.__aDict__)
-          self.eDict = copy.deepcopy(self.parse.Events)
-          self.species = copy.deepcopy(self.species_stochmatrix)
-          self.species += [species for species in sorted(self.aDict)]
-          self.species_pos  = {}
+          self.eDict = copy.deepcopy(self.parse.Mod.__eDict__)
+          self.species_names = copy.deepcopy(self.species_stochmatrix)
+          self.species_names += [species for species in sorted(self.aDict)]
+          self.species_pos = {}
           i=0
-          for species in self.species:
+          for species in self.species_names:
               self.species_pos[species] = i # Determine once for each species the position in the X-matrix
               i+=1
           self.sim_dump = []
           self.n_reactions = len(self.propensities) # Number of reactions
-          self.sim_a_mu = np.zeros(self.n_reactions)  
+          self.sim_a_mu = np.zeros(self.n_reactions)
       except:
           self.IsExit = True
           print "Error: It is impossible to parse the input file:", self.ModelFile, "from directory" , self.ModelDir
 
-
-  def Execute(self,settings):       
+  def Execute(self,settings):
     """
     Generates T trajectories of the Markov jump process.
 
@@ -110,14 +109,18 @@ class NextReactionMethod(PySCeS_Connector):
     self.X_matrix = copy.deepcopy(settings.X_matrix)
     self.IsTrackPropensities = copy.copy(settings.IsTrackPropensities)
     self.sim_t = copy.copy(settings.starttime)
-    self.GetEventAtTime()
-    self.GetEventAtAmount()
     self.BuildInits()
-    self.Propensities()
-    if not self.sim_t:
-        self.timestep = 1      
-        self.Initial_Conditions()
+    self.events = copy.copy(self.parse.Mod.__events__)
+    IsPerformEvent = False
+    for ev in self.events:
+        for s_id in sorted(self.species_names, reverse=True): # makes sure that the longest identifiers are replaced first
+            ev.code_string = ev.code_string.replace('self.mod.%s'% s_id,'X_matrix[%s]' % self.species_pos[s_id]) 
+        ev.xcode = compile("self.state = %s" % ev.code_string,'event%s' % ev,'exec')   
     
+    self.Propensities()
+    if not self.sim_t:      
+        self.timestep = 1
+        self.Initial_Conditions()    
     #####################
     if self.aDict != {} and not self.sim_t:
         self.AssignmentRules()
@@ -128,42 +131,44 @@ class NextReactionMethod(PySCeS_Connector):
     #####################
     while self.sim_t < settings.endtime and self.timestep < settings.timesteps:
         self.sim_a_0 = self.sim_a_mu.sum()
-        if self.sim_a_0 <= 0:                               # All reactants got exhausted
-             if settings.endtime < settings.timesteps:
-                 last_output = self.sim_output[-1]
-                 last_output[0] = settings.endtime
-                 self.sim_output.append(last_output)                
-             break            
-                 
+        if self.sim_a_0 <= 0:                         # All reactants got exhausted
+             break                 
               
-        self.RunTimeEvent(settings)                         # Run next reaction method
-        if self.sim_t > self.event_time and self.event_time != None:
-            self.DoEvent()                                  # Time event
-            self.sim_t = copy.copy(self.event_time)
-            self.event_time = None          
-        if self.event_amount != None:                       # Amount Event
-            if self.X_matrix[self.event_amount_index] > self.event_amount:
-                self.DoEvent()
-            
-        for i in xrange(self.n_species):                    # Add 'dt' to a species amount:
-            try:
-                self.distributions[i][self.sim_output[-1][i+1]] += self.sim_t-self.sim_output[-1][0]
-            except KeyError:     
-                self.distributions[i][self.sim_output[-1][i+1]] = self.sim_t-self.sim_output[-1][0]                
-
+        self.RunExactTimestep(settings)                   # Run time step              
+        ### Events handling                     
+        for ev in self.events:
+            IsTrigger = ev(self.sim_t,self.X_matrix)
+            #print IsTrigger
+            if IsTrigger:
+                for s_id in sorted(self.eDict[ev.name]['assignments']):                        
+                    s_index = self.species_pos[s_id]
+                    self.X_matrix[s_index] = float(self.eDict[ev.name]['assignments'][s_id])                          
+                    self.sim_t = ev._ASS_TIME_
+                if '_TIME_' in ev.formula:            # Do only with time events
+                    ev.formula = ev.formula.replace(' ','')
+                    m = re.search('_TIME_,*\d+\.\d*|_TIME_,*\d+',ev.formula)                                      
+                    self.sim_t = float(m.group(0)[7:])
+                    #print self.sim_t
+                    self.events.remove(ev)            # Remove time events; they can happen only once per trajectory
+                IsPerformEvent = True 
         ### Start output Generation ###            
-        event_output = list(self.X_matrix)            
-        event_output += [amount for amount in self.fixed_species_amount]                
+        timestep_output = list(self.X_matrix)            
+        timestep_output += [amount for amount in self.fixed_species_amount]                
         if self.aDict != {}:
             self.AssignmentRules()
-            event_output += [value for value in self.aDict_species]
-        event_output.append(self.reaction_index+1)	
-        event_output.insert(0,self.sim_t)
-        self.sim_output.append(event_output)
+            timestep_output += [value for value in self.aDict_species]
+        timestep_output.append(self.reaction_index+1)	
+        timestep_output.insert(0,self.sim_t)
+        self.sim_output.append(timestep_output)
         ###  End output Generation  ###
-        if self.sim_t < settings.endtime and self.timestep < settings.timesteps:
-            self.sim_a_mu_prev = copy.copy(self.sim_a_mu)                 
-            self.to_update = self.reagents[self.reaction_index] # Determine vars to update
+        
+        if (self.sim_t < settings.endtime) and (self.timestep < settings.timesteps):
+            self.sim_a_mu_prev = copy.copy(self.sim_a_mu)    
+            if not IsPerformEvent:             
+                self.species_to_update = self.reagents[self.reaction_index] # Determine vars to update
+            else:
+                self.species_to_update  = [s for s in xrange(self.n_species)]                
+                IsPerformEvent = False   
             self.Propensities()                                 # Update Propensities    
             t_prev = self.sim_output[self.timestep-1][0]        
             for j in self.prop_to_update:
@@ -184,8 +189,7 @@ class NextReactionMethod(PySCeS_Connector):
         if self.IsTrackPropensities:
             a_ravel = list(self.sim_a_mu.ravel())
             a_ravel.insert(0,self.sim_t)
-            self.propensities_output.append(a_ravel)
-                  
+            self.propensities_output.append(a_ravel)                  
 
   def Initial_Conditions(self):                
       """ This function initiates the output format with the initial concentrations  """
@@ -198,11 +202,8 @@ class NextReactionMethod(PySCeS_Connector):
 
       output_init = [self.sim_t]      
       for init in self.X_matrix:                              # Output at t = 0 
-          if init < 0:
-              print "Error: There are initial negative concentrations!"
-              sys.exit()
+          assert init >= 0, "Error: There are initial negative concentrations!"
           output_init.append(int(init))
-
       for amount in self.fixed_species_amount:
           output_init.append(amount)
       self.aDict_indices = []
@@ -211,42 +212,7 @@ class NextReactionMethod(PySCeS_Connector):
                output_init.append(self.parse.Mod.__pDict__[species]['initial'])
                self.aDict_indices.append(len(output_init)-1)
       output_init.append(np.NAN)
-      self.sim_output.append(output_init)
-      self.distributions = [{} for i in xrange(self.n_species)]       
-
-  def GetEventAtTime(self):
-      """ Get times where events happen"""
-      self.event_time = None
-      if 'reset' in sorted(self.eDict):
-          if 'TIME' in self.eDict['reset']['trigger']:
-              m = re.search(', *\d+\.\d*',self.eDict['reset']['trigger'])
-              if not m:
-                 m = re.search(', *\d+',self.eDict['reset']['trigger'])
-              self.event_time = float(m.group(0)[1:])
-
-  def GetEventAtAmount(self):
-      """ Get amount where events happen"""
-      self.event_amount = None
-      if 'reset' in sorted(self.eDict):
-          i=0
-          for species in self.species_stochmatrix:              
-              if re.search('\( *'+species+' *,',self.eDict['reset']['trigger']):
-                  m = re.search(', *\d+\.\d*',self.eDict['reset']['trigger'])
-                  if not m:
-                     m = re.search(', *\d+',self.eDict['reset']['trigger'])
-                  self.event_amount = float(m.group(0)[1:])
-                  self.event_amount_index = i
-                  break
-              i+=1
-
-  def DoEvent(self): 
-      """ Do the event of the model """
-      i=0
-      for species in self.species_stochmatrix:
-         if species in self.eDict['reset']['assignments']:
-             self.X_matrix[i] = float(self.eDict['reset']['assignments'][species])                     
-             if i not in self.to_update: self.to_update.append(i)   # update species value for propensities for next reaction fire
-         i+=1
+      self.sim_output.append(output_init)     
 
   def AssignmentRules(self):
        """ Builds the assignment rules """ 
@@ -289,36 +255,37 @@ class NextReactionMethod(PySCeS_Connector):
       """
       Determines the propensities to fire for each reaction at the current time point. At t=0, all the rate equations are compiled. 
       """   
-      if self.IsInit:			                     # Compile rate-eqs
+      if self.IsInit:			                        # Compile rate-eqs
         code_str = """"""
-        self.sim_a_mu = np.zeros([self.n_reactions])         # Initialize a(mu)
+        self.sim_a_mu = np.zeros([self.n_reactions])            # Initialize a(mu)
         for i in xrange(self.n_reactions):        
             code_str += "if self.rFire[%s]: r_vec[%s]=%s\n" % (i,i,self.propensities[i])
         self.req_eval_code = compile(code_str,"RateEqEvaluationCode","exec")
         [setattr(__species__,self.species_stochmatrix[s],self.X_matrix[s]) for s in xrange(self.n_species)] # Set species quantities
-        self.IsInit = False
+        self.IsInit = False     
+        #print code_str   
       else:
-          [setattr(__species__,self.species_stochmatrix[s],self.X_matrix[s]) for s in self.to_update]
-      self.rateFunc(self.req_eval_code,self.sim_a_mu)        # Calc. Propensities 
-      if self.sim_a_mu.min() < 0:
-          print "Error: Negative propensities are found"
-      else: self.sim_a_mu = abs(self.sim_a_mu)
+          [setattr(__species__,self.species_stochmatrix[s],self.X_matrix[s]) for s in self.species_to_update]
+          #for s in self.species_to_update: print s
+      self.rateFunc(self.req_eval_code,self.sim_a_mu)           # Calc. Propensities 
+      assert self.sim_a_mu.min() >= 0, "Error: Negative propensities are found" 
+      self.sim_a_mu = abs(self.sim_a_mu)                                    # -0 to 0
 
-  def RunTimeEvent(self,settings):
+  def RunExactTimestep(self,settings):
       """ Perform a direct SSA time step and pre-generate M random numbers """ 
       #np.random.seed(5) 
       if self.sim_t == 0:       
-          randoms = np.random.random(self.n_reactions) # Pre-generate for each time step M random numbers
+          randoms = np.random.random(self.n_reactions)          # Pre-generate for each time step M random numbers
           self.randoms_log_init = np.log(randoms)*-1
           self.InitialStep()
           self.count  = 0
-          randoms = np.random.random(1000)                  # Create M random numbers
+          randoms = np.random.random(1000)                      # Create M random numbers
           self.randoms_log = np.log(randoms)*-1
           
       self.rFire= np.zeros(self.n_reactions) 
       minimum = self.tree.heap[0]
-      self.reaction_index = minimum[1]                      # Pick reaction to executeO(1)
-      tau = minimum[0]                                      # Pick tau O(1)     
+      self.reaction_index = minimum[1]                          # Pick reaction to executeO(1)
+      tau = minimum[0]                                          # Pick tau O(1)     
       if tau < settings.endtime:
           self.sim_t = tau                                      # New time  
           self.prop_to_update = self.dep_graph[self.reaction_index] # Propensities to update      
@@ -332,8 +299,7 @@ class NextReactionMethod(PySCeS_Connector):
               self.timestep += 1
           except MemoryError,ex:
               print ex
-              sys.exit()      
-      
+              sys.exit()
       else: 
           self.sim_t = settings.endtime
           self.reaction_index = np.nan
@@ -347,40 +313,42 @@ class NextReactionMethod(PySCeS_Connector):
           temp = (self.sim_taus[j],j)
           pairs.append((self.sim_taus[j],j))
           j+=1 	
-      self.tree = BinaryHeap(pairs)                         # Create binary tree    
+      self.tree = BinaryHeap(pairs)                           # Create binary tree    
       ##show_tree(self.tree.heap)      
       
   def UpdateHeap(self):       
     """ This function calculates new tau values for reactions that are changed. After this calculations, the binary heap is updated, which is much more efficient then rebuilding the entire heap. """
-    for j in self.prop_to_update:
     #for j in xrange(self.n_reactions): 
-      location = self.tree.index[j]
-      if j == self.reaction_index:                          # alpha = mu
-          if self.count == 999:
-              randoms = np.random.random(1000)              # Create M random numbers
-              self.randoms_log = np.log(randoms)*-1
-              self.count = 0				
-          if self.sim_a_mu[j] != 0:
-              tau = self.randoms_log[self.count]/self.sim_a_mu[j] + self.sim_t
-              self.count +=1
-          else: tau = 1000e1000
-          self.sim_taus[j] = tau
-          self.tree.heap[location] = (tau,j)
-      else:                                                 # alpha != mu
-          last_pos_a_mu = self.sim_a_mu_zero[j]['a_alpha_old']
-          t1 = self.sim_a_mu_zero[j]['t1']
-          if self.sim_a_mu_prev[j] != 0 and last_pos_a_mu !=0: # a_j > 0
-              tau_alpha = self.sim_taus[j]
-              tau = (float(self.sim_a_mu_prev[j])/self.sim_a_mu[j])*(tau_alpha-self.sim_t)+self.sim_t
-          elif self.sim_a_mu_prev[j] == 0 and last_pos_a_mu == 0 and t1 == 0:   # a_j zero from start
-              tau = self.randoms_log_init[j]/self.sim_a_mu[j] + self.sim_t
-          else:                                             # a_j became zero after some time t1
-              tau_alpha = self.sim_a_mu_zero[j]['tau_alpha']          
-              tau = (float(last_pos_a_mu)/self.sim_a_mu[j])*(tau_alpha-t1)+self.sim_t
-          self.sim_taus[j] = tau
-          self.tree.heap[location] = (tau,j)
-      self.tree.Done = 0
-      while not self.tree.Done: self.tree.Update_AUX(tau,j) 
+    for j in self.prop_to_update:      
+        location = self.tree.index[j]
+        if j == self.reaction_index:                          # alpha = mu
+            if self.count == 999:
+                randoms = np.random.random(1000)              # Create M random numbers
+                self.randoms_log = np.log(randoms)*-1
+                self.count = 0				
+            if self.sim_a_mu[j] != 0:
+                tau = self.randoms_log[self.count]/self.sim_a_mu[j] + self.sim_t
+                self.count +=1
+            else:
+                tau = 1000e1000
+            self.sim_taus[j] = tau
+            self.tree.heap[location] = (tau,j)
+        else:                                                 # alpha != mu
+            last_pos_a_mu = self.sim_a_mu_zero[j]['a_alpha_old']
+            t1 = self.sim_a_mu_zero[j]['t1']
+            if (self.sim_a_mu_prev[j] != 0) and (last_pos_a_mu !=0): # a_j > 0
+                tau_alpha = self.sim_taus[j]
+                tau = (float(self.sim_a_mu_prev[j])/self.sim_a_mu[j])*(tau_alpha-self.sim_t)+self.sim_t
+            elif (self.sim_a_mu_prev[j] == 0) and (last_pos_a_mu == 0) and (t1 == 0):   # a_j zero from start
+                tau = self.randoms_log_init[j]/self.sim_a_mu[j] + self.sim_t
+            else:                                             # a_j became zero after some time t1
+                tau_alpha = self.sim_a_mu_zero[j]['tau_alpha']          
+                tau = (float(last_pos_a_mu)/self.sim_a_mu[j])*(tau_alpha-t1)+self.sim_t
+            self.sim_taus[j] = tau
+            self.tree.heap[location] = (tau,j)
+        self.tree.Done = 0
+        while not self.tree.Done:
+            self.tree.Update_AUX(tau,j) 
 
   def BuildInits(self):
       """ Build initials that are necessary to generate a trajectory """    
@@ -388,5 +356,5 @@ class NextReactionMethod(PySCeS_Connector):
       self.sim_a_mu_zero = []
       for i in xrange(self.n_reactions):
           self.sim_a_mu_zero.append({'t1':0,'a_alpha_old':0,'tau_alpha':0})    
-      self.sim_taus = None	 	 			            # necessary for iteration
-      self.tree = None  					            # necessary for iteration
+      self.sim_taus = None	 	 		      # necessary for iteration
+      self.tree = None  			              # necessary for iteration
