@@ -7,9 +7,9 @@ This program performs Optimized Explicit Tau-leaping algorithm, which is an appr
 
 [1] Cao. Y, Gillespie D., Petzold L. (2006), "Efficient step size selection for the tau-leaping method", J.Chem. Phys. 28:124-135
 
-Written by TR Maarleveld, Amsterdam, The Netherlands
+Written by T.R. Maarleveld, Amsterdam, The Netherlands
 E-mail: tmd200@users.sourceforge.net
-Last Change: July 4, 2012
+Last Change: April 08, 2013
 """
 
 ############################## IMPORTS ###################################
@@ -81,12 +81,12 @@ class OTL(PySCeS_Connector):
         self.fixed_species_amount = copy.deepcopy(self.parse.fixed_species_amount)
         self.rate_eqs = copy.deepcopy(self.parse.rate_eqs)
         self.aDict = copy.deepcopy(self.parse.Mod.__aDict__)
-        self.eDict = copy.deepcopy(self.parse.Events)
-        self.species = copy.deepcopy(self.species_stochmatrix)
-        self.species += [species for species in sorted(self.aDict)]
+        self.eDict = copy.deepcopy(self.parse.Mod.__eDict__)
+        self.species_names = copy.deepcopy(self.species_stochmatrix)
+        self.species_names += [species for species in sorted(self.aDict)]
         self.species_pos  = {}
         i=0
-        for species in self.species:
+        for species in self.species_names:
             self.species_pos[species] = i # Determine once for each species the position in the X-matrix
             i+=1
         self.sim_dump = [] 
@@ -107,12 +107,6 @@ class OTL(PySCeS_Connector):
     self.X_matrix = copy.deepcopy(settings.X_matrix)
     self.IsTrackPropensities = copy.copy(settings.IsTrackPropensities)
     self.sim_t = copy.copy(settings.starttime)
-    self.GetEventAtTime()
-    self.GetEventAtAmount()
-    self.Propensities()
-    if not self.sim_t:
-        self.timestep = 1      
-        self.Initial_Conditions()   
     
     self.sim_epsilon = epsilon
     self.sim_Nc = 10
@@ -131,10 +125,19 @@ class OTL(PySCeS_Connector):
     IsFinished = False
     IsNegative = False
     tauleapsteps = 0
-    self.GetEventAtTime()
-    self.GetEventAtAmount()
+    (orders,hors,HO_info) = DetermineOrderHOR(self.rate_eqs,self.reactants)    
+    
+    self.events = copy.copy(self.parse.Mod.__events__)
+    IsPerformEvent = False
+    for ev in self.events:
+        for s_id in sorted(self.species_names, reverse=True): # makes sure that the longest identifiers are replaced first
+            ev.code_string = ev.code_string.replace('self.mod.%s'% s_id,'X_matrix[%s]' % self.species_pos[s_id]) 
+        ev.xcode = compile("self.state = %s" % ev.code_string,'event%s' % ev,'exec')   
+    
     self.Propensities()
-    (orders,hors,HO_info) = DetermineOrderHOR(self.rate_eqs,self.reactants)
+    if not self.sim_t:      
+        self.timestep = 1
+        self.Initial_Conditions()      
     #####################
     if self.aDict != {} and not self.sim_t:
         self.AssignmentRules()
@@ -162,29 +165,33 @@ class OTL(PySCeS_Connector):
             if not self.IsNegative:                
                 self.sim_t += self.sim_tau
                 IsNegative = False
-                if self.sim_t > self.event_time and self.event_time != None:
-                    self.DoEvent()
-                    self.sim_t = copy.copy(self.event_time)
-                    self.event_time = None
-                if self.event_amount != None:
-                    if self.X_matrix[self.event_amount_index] > self.event_amount:
-                        self.DoEvent()            
-                for i in xrange(self.n_species):                # Add 'dt' to a species amount:
-                    try:
-                        self.distributions[i][self.sim_output[-1][i+1]] += self.sim_t - self.sim_output[-1][0]
-                    except KeyError:
-                        self.distributions[i][self.sim_output[-1][i+1]] = self.sim_t - self.sim_output[-1][0]
+                ### Events handling                       
+                for ev in self.events:
+                    IsTrigger = ev(self.sim_t,self.X_matrix)
+                    #print IsTrigger
+                    if IsTrigger:
+                        for s_id in sorted(self.eDict[ev.name]['assignments']):                        
+                            s_index = self.species_pos[s_id]
+                            self.X_matrix[s_index] = float(self.eDict[ev.name]['assignments'][s_id])                          
+                            self.sim_t = ev._ASS_TIME_
+                        if '_TIME_' in ev.formula:            # ONLY WITH TIME EVENTS
+                            ev.formula = ev.formula.replace(' ','')
+                            m = re.search('_TIME_,*\d+\.\d*|_TIME_,*\d+',ev.formula)                                      
+                            self.sim_t = float(m.group(0)[7:])
+                            #print self.sim_t
+                            self.events.remove(ev)            # Remove time events; they can happen only once per trajectory
+                        IsPerformEvent = True
                         
                 ### Start output Generation ###     
-                event_output = list(self.X_matrix)            
-                event_output += [amount for amount in self.fixed_species_amount]                
+                timestep_output = list(self.X_matrix)            
+                timestep_output += [amount for amount in self.fixed_species_amount]                
                 if self.aDict != {}:
                     self.AssignmentRules()
-                    event_output += [value for value in self.aDict_species]
-                event_output.insert(0,self.sim_t)           
+                    timestep_output += [value for value in self.aDict_species]
+                timestep_output.insert(0,self.sim_t)           
                 ###  End output Generation  ### 
                 self.Propensities()
-                self.sim_output.append(event_output)       
+                self.sim_output.append(timestep_output)       
                 if self.IsTrackPropensities:                
                     a_ravel = list(self.sim_a_mu.ravel())
                     a_ravel.insert(0,self.sim_t)
@@ -194,51 +201,55 @@ class OTL(PySCeS_Connector):
                 IsNegative = True
                 self.sim_tau_prime = self.sim_tau_prime/2.0   
         elif self.IsExact:                                      # Direct SSA step
-            m = 0
+            i = 0
             ExactTimesteps = 100       
-            while m < ExactTimesteps and self.sim_t < settings.endtime and self.timestep < settings.timesteps:               
+            while i < ExactTimesteps and self.sim_t < settings.endtime and self.timestep < settings.timesteps:               
                 self.sim_a_0 = self.sim_a_mu.sum() 
                 if self.sim_a_0 <= 0:                           # All reactants got exhausted
                     IsFinished = True           
                     break
                     
-                self.RunExactSSA()                
-                if self.sim_t > self.event_time and self.event_time != None:
-                    self.DoEvent()                              # Time Event
-                    self.sim_t = copy.copy(self.event_time)
-                    self.event_time = None                
-                if self.event_amount != None:                   # Amount Event
-                    if self.X_matrix[self.event_amount_index] > self.event_amount:
-                        self.DoEvent()                
-                for i in xrange(self.n_species):                # Add 'dt' to a species amount:
-                    try:
-                        self.distributions[i][self.sim_output[-1][i+1]] += self.sim_t - self.sim_output[-1][0]
-                    except KeyError:
-                        self.distributions[i][self.sim_output[-1][i+1]] = self.sim_t - self.sim_output[-1][0]
- 
+                self.RunExactTimestep()                
+                ### Events handling                       
+                for ev in self.events:
+                    IsTrigger = ev(self.sim_t,self.X_matrix)
+                    #print IsTrigger
+                    if IsTrigger:
+                        for s_id in sorted(self.eDict[ev.name]['assignments']):                        
+                            s_index = self.species_pos[s_id]
+                            self.X_matrix[s_index] = float(self.eDict[ev.name]['assignments'][s_id])                          
+                            self.sim_t = ev._ASS_TIME_
+                        if '_TIME_' in ev.formula:            # ONLY WITH TIME EVENTS
+                            ev.formula = ev.formula.replace(' ','')
+                            m = re.search('_TIME_,*\d+\.\d*|_TIME_,*\d+',ev.formula)                                      
+                            self.sim_t = float(m.group(0)[7:])
+                            #print self.sim_t
+                            self.events.remove(ev)            # Remove time events; they can happen only once per trajectory
+                        IsPerformEvent = True
+
                 ### Start output Generation ###
-                event_output = list(self.X_matrix)            
-                event_output += [amount for amount in self.fixed_species_amount]                
+                timestep_output = list(self.X_matrix)            
+                timestep_output += [amount for amount in self.fixed_species_amount]                
                 if self.aDict != {}:
                     self.AssignmentRules()
-                    event_output += [value for value in self.aDict_species]
-                event_output.insert(0,self.sim_t)
+                    timestep_output += [value for value in self.aDict_species]
+                timestep_output.insert(0,self.sim_t)
                 ###  End output Generation  ###
-                if m < ExactTimesteps:
+                if i < ExactTimesteps:
                     self.timestep +=1
-                    self.sim_output.append(event_output)                    
+                    self.sim_output.append(timestep_output)                    
                     self.Propensities()
                     if self.IsTrackPropensities:
                         a_ravel = list(self.sim_a_mu.ravel())
                         a_ravel.insert(0,self.sim_t)
                         self.propensities_output.append(a_ravel)
-                m+=1                
+                i+=1                
         #################### End possible feedback loop #################                 
         self.timestep +=1
         if IsFinished:
             break       
 
-  def RunExactSSA(self):
+  def RunExactTimestep(self):
       """ Perform a direct method SSA time step"""
       if self.count == 1000:
           randoms = np.random.random(1000)
@@ -450,9 +461,7 @@ class OTL(PySCeS_Connector):
 
       output_init = [self.sim_t]      
       for init in self.X_matrix:                             # Output at t = 0 
-          if init < 0:
-              print "Error: There are initial negative concentrations!"
-              sys.exit()
+          assert init >= 0, "Error: There are initial negative concentrations!"       
           output_init.append(int(init))
 
       for amount in self.fixed_species_amount:
@@ -463,40 +472,6 @@ class OTL(PySCeS_Connector):
                output_init.append(self.parse.Mod.__pDict__[species]['initial'])
                self.aDict_indices.append(len(output_init)-1)
       self.sim_output.append(output_init)
-      self.distributions = [{} for i in xrange(self.n_species)]
-
-  def GetEventAtTime(self):
-      """ Get times where events happen"""
-      self.event_time = None
-      if 'reset' in sorted(self.eDict):
-          if 'TIME' in self.eDict['reset']['trigger']:
-              m = re.search(', *\d+\.\d*',self.eDict['reset']['trigger'])
-              if not m:
-                 m = re.search(', *\d+',self.eDict['reset']['trigger'])
-              self.event_time = float(m.group(0)[1:])
-
-  def GetEventAtAmount(self):
-      """ Get amount where events happen"""
-      self.event_amount = None
-      if 'reset' in sorted(self.eDict):
-          i=0
-          for species in self.species_stochmatrix:              
-              if re.search('\( *'+species+' *,',self.eDict['reset']['trigger']):
-                  m = re.search(', *\d+\.\d*',self.eDict['reset']['trigger'])
-                  if not m:
-                     m = re.search(', *\d+',self.eDict['reset']['trigger'])
-                  self.event_amount = float(m.group(0)[1:])
-                  self.event_amount_index = i
-                  break
-              i+=1
-
-  def DoEvent(self): 
-     """ Do the event of the model """
-     i=0
-     for species in self.species_stochmatrix:
-         if species in self.eDict['reset']['assignments']:
-             self.X_matrix[i] = float(self.eDict['reset']['assignments'][species])           
-         i+=1
 
   def AssignmentRules(self):
        """ Builds the assignment rules """ 
@@ -548,10 +523,8 @@ class OTL(PySCeS_Connector):
       else:     
           [setattr(__species__,self.species_stochmatrix[s],self.X_matrix[s]) for s in xrange(self.n_species)] # Set species quantities  
       self.rateFunc(self.req_eval_code,self.sim_a_mu)       # Calc. Propensities
-      if self.sim_a_mu.min() < 0:
-          print "Error: Negative propensities are found"
-      else: 
-          self.sim_a_mu = abs(self.sim_a_mu)   
+      assert self.sim_a_mu.min() >= 0, "Error: Negative propensities are found" 
+      self.sim_a_mu = abs(self.sim_a_mu)                                    # -0 to 0
  
 
 ################### Useful functions #########################
