@@ -4,16 +4,16 @@ PySCeS MDL Parser
 =================
 The PySCeS parser is used to import a model written in the MDL of PySCeS. Further, all required input do to stochastic simulations is build.
 
-Written by Timo Maarleveld, Amsterdam, The Netherlands
+Written by T.R. Maarleveld, Amsterdam, The Netherlands
 E-mail: tmd200@users.sourceforge.net
-Last Change: March 16, 2012
+Last Change: May 17, 2013
 """
 
-import os, copy, time,sys
+import os,copy,time,sys,operator
 try: 
     import numpy as np
-except:
-    print "Error: the NumPy module is not installed"
+except ImportError:
+    print "Error: The NumPy module is not installed"
     sys.exit()
   
 import stochpy.modules.PyscesParse
@@ -21,9 +21,177 @@ import stochpy.modules.SBML2PSC as SBML2PSC
 from stochpy import model_dir, output_dir
 from stochpy.core2.InfixParser import MyInfixParser
 
+InfixParser = MyInfixParser()
+InfixParser.buildlexer()
+InfixParser.buildparser(debug=0, debugfile='infix.dbg', tabmodule='infix_tabmodule')
+InfixParser.setNameStr('self.', '')
+
 mach_spec = np.MachAr()
 pscParser = stochpy.modules.PyscesParse.PySCeSParser(debug=0)
 
+class NewCoreBase(object):
+    """
+    Core2 base class, needed here as we use Core2 derived classes
+    in PySCes
+    """
+    name = None
+    __DEBUG__ = False
+
+    def getName(self):
+        return self.name
+
+    def setName(self,name):
+        self.name = name
+
+    def get(self, attr):
+        """Return an attribute whose name is str(attr)"""
+        return self.__getattribute__(attr)
+        
+# this must stay in sync with core2
+class NumberBase(NewCoreBase):
+    """
+    Derived Core2 number class.
+    """
+    value = None
+    value_initial = None
+
+    def __call__(self):
+        return self.value
+
+    def getValue(self):
+        return self.value
+
+    def setValue(self, v):
+        self.value = v
+        
+class EventAssignment(NumberBase):
+    """
+    Event assignments are actions that are triggered by an event.
+    Ported from Core2 to build an event handling framework fro PySCeS
+    """
+    variable = None
+    symbols = None
+    formula = None
+    code_string = None
+    xcode = None
+    mod = None
+    piecewises = None
+    __DEBUG__ = False
+
+    def __call__(self):
+        setattr(self.mod, self.variable, self.value)
+        if self.__DEBUG__: print '\tAssigning %s = %s' % (self.variable, self.value)
+        return True
+
+    def __init__(self, name, mod):
+        self.setName(name)
+        self.mod = mod
+
+    def setVariable(self, var):
+        self.variable = var
+
+    def setFormula(self, formula):
+        self.formula = formula
+        InfixParser.setNameStr('self.mod.', '')
+        InfixParser.SymbolReplacements = {'_TIME_':'self.mod._TIME_'} # TODO
+        InfixParser.parse(formula)
+        self.piecewises = InfixParser.piecewises
+        self.symbols = InfixParser.names
+        self.code_string = 'self.value=%s' % InfixParser.output
+        self.xcode = compile(self.code_string, 'EvAs: %s' % self.name, 'exec')
+        ##  print '\t', self.name, self.code_string
+
+    def evaluateAssignment(self):
+        exec(self.xcode)
+
+# adapted from core2
+class Event(NewCoreBase):
+    """
+    Event's have triggers and fire EventAssignments when required.
+    Ported from Core2.
+    """
+    trigger = None
+    delay = 0.0
+    formula = None
+    code_string = None
+    xcode = None
+    state0 = False
+    state = False
+    assignments = None
+    _TIME_ = 0.0
+    _ASS_TIME_ = 0.0   
+    _TRIGGER_TIME_ = False
+    _need_action = False
+    symbols = None
+    _time_symbol = None
+    piecewises = None
+    mod = None
+    __DEBUG__ = True
+
+    def __init__(self, name, mod):
+        self.setName(name)
+        self.assignments = []
+        self.mod = mod
+
+    def __call__(self, time,X_matrix):#,__species__):        
+        self._TIME_ = time
+        exec(self.xcode)       
+        #print self._TIME_,self.state,__species__.P2
+        ret = False
+        if self.state0 and not self.state:
+            self.state0 = self.state
+        if not self.state0 and self.state:
+            #for ass in self.assignments:
+            #    ass.evaluateAssignment()
+            self.state0 = self.state
+            self._need_action = True
+            self._ASS_TIME_ = time + self.delay
+            #print self._ASS_TIME_            
+            #print '\nevent %s is evaluating at %s' % (self.name, time)
+            ret = False
+        if self._need_action and self._TIME_ >= self._ASS_TIME_:
+            #for ass in self.assignments:
+            #    ass()
+            #print 'event %s is assigning at %s (delay=%s)' % (self.name, time, self.delay)
+            self._need_action = False
+            ret = True
+        return ret
+
+    def setTrigger(self, formula, delay=0.0):
+        self.formula = formula
+        self.delay = delay
+        InfixParser.setNameStr('self.mod.', '')
+        ##  print formula, delay
+        if self._time_symbol != None:
+            InfixParser.SymbolReplacements = {self._time_symbol : '_TIME_'}
+        else:
+            InfixParser.SymbolReplacements = {'_TIME_' : '_TIME_'}
+        InfixParser.parse(formula)
+        self.piecewises = InfixParser.piecewises
+        self.symbols = InfixParser.names
+        self.code_string = 'self.state=%s' % InfixParser.output
+        #self.code_string = InfixParser.output
+        #self.code_string = self.code_string.replace('self.mod._TIME_','self.sim_t')
+        self.code_string = self.code_string.replace('self.mod._TIME_','self._TIME_')
+        #self.code_string = 'IsTrigger = ' + self.code_string # TODO
+        #self.code_string = self.code_string.replace('self.mod.','__species__.')        
+        #self.xcode = compile(self.code_string, 'Ev: %s' % self.name, 'exec')
+        ##  print self.name, self.code_string
+
+    def setAssignment(self, var, formula):
+        ass = EventAssignment(var, mod=self.mod)
+        ass.setVariable(var)
+        ass.setFormula(formula)
+        self.assignments.append(ass)
+        self.__setattr__('_'+var, ass)
+
+    def reset(self):
+        self.state0 = False
+        self.state = False
+        self._TIME_ = 0.0
+        self._ASS_TIME_ = 0.0        
+        self._TRIGGER_TIME_ = False
+
 class NewCoreBase(object):
     """ Core2 base class, needed here as we use Core2 derived classes in PySCeS """
     name = None
@@ -94,7 +262,8 @@ class PyscesInputFileParser(object):
         else:
             assert os.path.exists(output_dir),"n&s is not a valid path" % output_dir
         self.__settings__ = {}
-        self.InitialiseInputFile()        
+        self.InitialiseInputFile()    
+        self.InitialiseEvents()    
         #self.Nmatrix = self.buildN()    # Stoch Matrix
 
     def InitialiseInputFile(self):
@@ -107,7 +276,8 @@ class PyscesInputFileParser(object):
         except:
             print 'Warning: Problem verifying: ' + os.path.join(self.ModelDir,self.ModelFile)
         
-        if self.ModelFile[-4:] == '.psc': pass
+        if self.ModelFile[-4:] == '.psc':
+            pass
         elif self.ModelFile[-4:].lower() == '.xml':
             try:
                 print "Info: extension is .xml"
@@ -188,7 +358,13 @@ class PyscesInputFileParser(object):
             if len(sorted(self.__compartments__)) > 0:
                 self.__HAS_COMPARTMENTS__ = True
             else:
-                self.__HAS_COMPARTMENTS__ = False
+                self.__HAS_COMPARTMENTS__ = False                
+                
+            if  self.__aDict__ != {}:
+                self.__HAS_ASSIGNMENTS__ = True
+                print 'Assignement(s) detected.'
+            else:  
+                self.__HAS_ASSIGNMENTS__ = False
 
             # no (self.)
             self.__fixed_species__ = copy.copy(pscParser.fixed_species)
@@ -198,14 +374,13 @@ class PyscesInputFileParser(object):
             self.__parameters__ = copy.copy(pscParser.parameters)
             #print "Parms",self.__parameters__
             self.__reactions__ = copy.copy(pscParser.reactions)
-	        #print "Reactions--> rate_eqs",self.__reactions__
+	    #print "Reactions--> rate_eqs",self.__reactions__
             self.__modifiers__ = copy.copy(pscParser.modifiers)
             #print self.__modifiers__
             self.__functions__ = pscParser.Functions.copy() 
             #print self.__functions__
             if self.__functions__ != {}:
-              print "Warning: The current version of StochPy does not support function input"
-
+                print "Warning: The current version of StochPy does not support function input"
         else:
             print '\nERROR: model parsing error, please check input file.\n'
         # added in a check for model correctness and human error reporting (1=ok, 0=error)
@@ -222,7 +397,31 @@ class PyscesInputFileParser(object):
                     print error[0] + '- ' + error[2][:20]
                 except:
                     print error
-            assert pscParser.ParseOK == 1, 'Input File Error'
+            assert pscParser.ParseOK == 1, 'Input File Error'            
+        
+    def InitialiseEvents(self):
+        """ Initialise Events """
+        self.__events__ = []
+        # for each event
+        for e in self.__eDict__:
+            ev = Event(e, self)
+            ev._time_symbol = self.__eDict__[e]['tsymb']
+            ev.setTrigger(self.__eDict__[e]['trigger'], self.__eDict__[e]['delay'])
+            # for each assignment
+            for ass in self.__eDict__[e]['assignments']:
+                ev.setAssignment(ass, self.__eDict__[e]['assignments'][ass])
+            self.__events__.append(ev)
+            setattr(self, ev.name, ev)
+            assert (not '_TIME_' in ev.formula or not ev.delay), "Error: The time event, %s, cannot have a delay" % ev.name            
+            #if ('_TIME_' in ev.formula and ev.delay):
+            #    print "Error: The time event, %s, cannot have a delay" % ev.name       
+            #    sys.exit()
+        os.chdir(self.ModelOutput)
+        if len(self.__events__) > 0:
+            self.__HAS_EVENTS__ = True
+            print 'Event(s) detected.'
+        else:
+            self.__HAS_EVENTS__ = False
 
 class PySCeS_Connector(PyscesInputFileParser):
   def __init__(self,ModelFile,ModelDir,IsNRM = False):
@@ -240,19 +439,19 @@ class PySCeS_Connector(PyscesInputFileParser):
         print er
         sys.exit()        
                  
-    self.buildN()
+    self.BuildN()
     if self.Mod.IsConverted:
         self.IsConverted = True  
     self.BuildX()
-    self.BuildReactions()    
+    self.BuildReactions()        
     if IsNRM:
         self.DetermineAffects()
         self.BuildDependencyGraph()
 
-  def buildN(self):
+  def BuildN(self):
       """ Generates the stoichiometric matrix N from the parsed model description. Returns a stoichiometric matrix (N) as a NumPy array """
       VarReagents = ['self.'+s for s in self.Mod.__species__]
-      self.N_matrix = np.zeros((len(VarReagents),len(self.Mod.__reactions__)))
+      self.N_matrix = np.zeros((len(VarReagents),len(self.Mod.__reactions__)))#,dtype=int)
       for reag in VarReagents:
           for id in self.Mod.__reactions__:
               if reag in sorted(self.Mod.__nDict__[id]['Reagents']):
@@ -266,20 +465,21 @@ class PySCeS_Connector(PyscesInputFileParser):
     self.fixed_species_amount = []
     for species in sorted(self.Mod.__sDict__):
         if species not in self.species:
-            amount = self.Mod.__sDict__[species]['initial']
+            s_amount = self.Mod.__sDict__[species]['initial']
             self.fixed_species.append(species)
-            self.fixed_species_amount.append(amount)
+            self.fixed_species_amount.append(s_amount)
    
-    self.X_matrix = np.zeros((n_species,1))        # ,dtype = int
-    self.Events = copy.copy(self.Mod.__eDict__)
-    for i in xrange(n_species):  
+    self.X_matrix = np.zeros((n_species,1))#dtype = int)    
+    for i in xrange(n_species):
         species = self.species[i]
         init_conc = self.Mod.__sDict__[species]['initial']
         if not self.Mod.__sDict__[species]['isamount']:    # Handle amount/concentration 19 September 2011
             if self.Mod.__compartments__:
                 compartments = sorted(self.Mod.__compartments__)
-                if len(compartments) > 1: print "Warning: Multiple compartments are detected"
-                if self.Mod.__compartments__[compartments[0]]['size'] != 1: print "Warning: species are given in concentrations and the volume is unequal to one, the species concentrations are multiplied by the compartment volume which will result in different numbers. This means that the rate equations also change."
+                if len(compartments) > 1:
+                    print "Warning: Multiple compartments are detected"
+                if self.Mod.__compartments__[compartments[0]]['size'] != 1:
+                    print "Warning: Species are given in concentrations and the volume is unequal to one, the species concentrations are multiplied by the compartment volume which will result in different numbers. This means that the rate equations also change."
                 init_conc *= self.Mod.__compartments__[compartments[0]]['size']
         self.X_matrix[i] = init_conc                       # Add initial conc.
 
@@ -289,70 +489,60 @@ class PySCeS_Connector(PyscesInputFileParser):
     self.reactants = []
     self.rate_eqs  = []
     self.propensities = []     
-    reaction_info = copy.copy(self.Mod.__nDict__)        # Reaction info from parser
+    reaction_info = copy.copy(self.Mod.__nDict__)         # Reaction info from parser
     self.reactions = copy.copy(self.Mod.__reactions__)
-    IsRever = False
-    for r in self.Mod.__reactions__:
-      temp_reactant = []
-      temp_product  = []      
-      rate_eq = reaction_info[r]['RateEq']              # Rate eq. info      
-      parms   = reaction_info[r]['Params']              # Parm from rate eq. 
-      for parm in parms:                                # Replace parm values (fixed)          
-          if parm[5:] in self.Mod.__parameters__:
-              parm_value = self.Mod.__pDict__[parm[5:]]['initial']             
-          elif parm[5:] in sorted(self.Mod.__compartments__):
-              parm_value = self.Mod.__compartments__[parm[5:]]['size']
-          rate_eq = rate_eq.replace(parm,str(parm_value))              
-      rate_eq = rate_eq.replace('self.','__species__.') # Make a special object 'species' where all species amounts are stored 20 Oct 2011        
-      self.propensities.append(rate_eq)                 # Add rate equation to props eq
-
-      init_reagents = sorted(reaction_info[r]['Reagents'])# Remove fixed species
-      reagents_dict = reaction_info[r]['Reagents']      
-
-      self.rate_eqs.append(reaction_info[r])
-      temp = []                                         # Convert reagents of each reaction into species indexes
-      for reagent in init_reagents:
-          try: 
-              temp.append(self.species.index(reagent[5:]))	# reagents of reaction r 16/09
-          except:
-              pass
-      self.reagents.append(temp)
+    IsReversible = False
+    for r_id in self.Mod.__reactions__:
+        temp_reactant = []
+        temp_product  = []      
+        rate_eq = reaction_info[r_id]['RateEq']           # Rate eq. info      
+        L_parms   = reaction_info[r_id]['Params']         # Parm from rate eq.         
+        for parm in L_parms:                              # Replace parm values (fixed)                  
+            parm = parm.replace('self.','')            
+            if parm in self.Mod.__parameters__:
+                if parm in sorted(self.Mod.__pDict__):
+                    parm_value = self.Mod.__pDict__[parm]['initial']             
+                elif parm in sorted(self.Mod.__sDict__): # Fix 28 Feb 2013
+                    parm_value = self.Mod.__sDict__[parm]['initial']
+            elif parm in sorted(self.Mod.__compartments__):
+                parm_value = self.Mod.__compartments__[parm]['size']
+            rate_eq = rate_eq.replace('self.%s' % parm,str(parm_value))
+        rate_eq = rate_eq.replace('self.','__species__.') # Make a special object 'species' where all species amounts are stored 20 Oct 2011        
+        self.propensities.append(rate_eq)                 # Add rate equation to props eq         
+        self.rate_eqs.append(reaction_info[r_id])
+        init_reagents = sorted(reaction_info[r_id]['Reagents'])# Remove fixed species      
+        L_s_indices = []                                  # Convert reagents of each reaction into species indexes
+        for reagent in init_reagents:
+            reagent = reagent.replace('self.','')
+            if reagent in self.species:
+                L_s_indices.append(self.species.index(reagent))	# reagents of reaction r 16/09
+        self.reagents.append(L_s_indices)
       
-      for var in init_reagents:	                        # Determine the reactants of each reaction
-          var = var.replace('self.','__species__.')          
-          if var+'*' in rate_eq:
-             temp_reactant.append(var[12:])          
-          elif var in rate_eq:
-             str_len = len(rate_eq)             
-             if str_len == (len(var) + rate_eq.index(var)):
-                 temp_reactant.append(var[12:])               
-      self.reactants.append(temp_reactant)      
-      if self.Mod.__nDict__[r]['Type'] == 'Rever':        
-          IsRever = True
-        
-    if IsRever:
-        print "Error: The model contains reversible reactions, while Stochastic Simulation Algorithms require irreversible reactions."    
-        sys.exit()
-
-    #print self.Mod.__nDict__
-    #try: #TODO
-    #self.propensities = Det2Stoch.Det2StochPropensities(self.propensities,self.Mod.__pDict__,self.rate_eqs,self.reactants)      
-    #except:
-    #    print "Warning: The model contains probably deterministic rate equations, but StochPy failed to convert them."
-    #    print "Info: Ignore this warning if you do not use mass-action-kinetics."
-    #print self.propensities
+        for var in init_reagents:	                  # Determine the reactants of each reaction
+            var = var.replace('self.','__species__.')          
+            if (var+'*') in rate_eq:
+               temp_reactant.append(var[12:])          
+            elif var in rate_eq:
+               str_len = len(rate_eq)             
+               if str_len == (len(var) + rate_eq.index(var)):
+                   temp_reactant.append(var[12:])   
+        self.reactants.append(temp_reactant)      
+        if self.Mod.__nDict__[r_id]['Type'] == 'Rever':        
+            IsReversible = True
+        assert not IsReversible, "Error: The model contains reversible reactions, while Stochastic Simulation Algorithms require irreversible reactions."
 
   def DetermineAffects(self):  
     """ Determine the affects for each reaction"""
     self.affects = [] 
     n=0    
-    for reaction in self.Mod.__reactions__:
-        reagents = self.Mod.__nDict__[reaction]['Reagents']
+    for r_id in self.Mod.__reactions__: 
+        D_reagents = self.Mod.__nDict__[r_id]['Reagents']
         n+=1
         temp_affects = []      
-        for species in reagents:            
-            if reagents[species] != 0:
-                temp_affects.append(species[5:])
+        for species in D_reagents:                   
+            if D_reagents[species] != 0:
+                s_id = species.strip('self.')  
+                temp_affects.append(s_id)
         self.affects.append(temp_affects)
 
   def BuildDependencyGraph(self):
@@ -364,25 +554,192 @@ class PySCeS_Connector(PyscesInputFileParser):
         for species_name in self.affects[i]:
             n=0
             for rate_eq in self.propensities:
-              species = '__species__.' + species_name                
-              if species + '*' in rate_eq:                  
-                  change.append(n)
-              elif species in rate_eq:
-                  str_len = len(rate_eq)
-                  #print rate_eq,str_len,var,(len(var) + rate_eq.index(var))
-                  if str_len == (len(species) + rate_eq.index(species)):
+                species = '__species__.' + species_name                
+                if species + '*' in rate_eq:                  
                     change.append(n)
-              n+=1                               
-        self.dep_graph.append(change)           
-    
+                elif species in rate_eq:
+                    str_len = len(rate_eq)
+                    #print rate_eq,str_len,var,(len(var) + rate_eq.index(var))
+                    if str_len == (len(species) + rate_eq.index(species)):
+                        change.append(n)
+                n+=1                               
+        self.dep_graph.append(change)        
 
-class InterpolatedDataObj(object):
-    def __init__(self):
-        self.species = []
-        self.points = None
-        self.means = None
-        self.standard_deviations = None
-        self.time = None 
+class RegularGridDataObj(object):    
+    """
+    This class is specifically designed to store the results of a stochastic time simulation on fixed time intervals
+    It has methods for setting the Time, Labels, Species and Propensity data and
+    getting Time, Species and Rate (including time) arrays. However, of more use:
+
+    - getOutput(\*args) feed this method species/rate labels and it will return
+      an array of [time, sp1, r1, ....]
+    - getDataAtTime(time) the data generated at time point "time".
+    - getDataInTimeInterval(time, bounds=None) more intelligent version of the above
+      returns an array of all data points where: time-bounds <= time <= time+bounds
+    """
+    time = None
+    species = None
+    propensities = None
+    species_autocorrelations = None 
+
+    species_means = None
+    species_standard_deviations = None
+    species_autocorrelations_means = None
+    species_autocorrelations_standard_deviations = None
+    propensities_means = None
+    propensities_standard_deviations = None
+    
+    HAS_SPECIES = False
+    HAS_PROPENSITIES = False
+    HAS_TIME = False  
+    HAS_SPECIES_AUTOCORRELATIONS = False
+    HAS_PROPENSITIES_AUTOCORRELATIONS = False
+    
+    HAS_SPECIES_AUTOCOVARIANCES = False
+    HAS_PROPENSITIES_AUTOCOVARIANCES = False
+    
+    HAS_AVERAGE_SPECIES_DISTRIBUTIONS = False
+    HAS_AVERAGE_PROPENSITIES_DISTRIBUTIONS = False    
+    
+    def setSpeciesDistributionAverage(self,mean,std):
+        """
+        Set means and stds of species data
+        
+        Input:
+         - *mean* (list)
+         - *std* (list)
+        """
+        self.species_distributions_means = mean
+        self.species_distributions_standard_deviations = std
+        self.HAS_AVERAGE_SPECIES_DISTRIBUTIONS = True
+
+    def setPropensitiesDistributionAverage(self,mean,std):
+        """
+        Set means and stds of species data
+
+        Input:
+         - *mean* (list)
+         - *std* (list)
+        """
+        self.propensities_distributions_means = mean
+        self.propensities_distributions_standard_deviations = std        
+        self.HAS_AVERAGE_PROPENSITIES_DISTRIBUTIONS = True
+    
+    def setSpecies(self, species, lbls=None):
+        """
+        Set the species array
+        
+        Input:
+         - *species* an array of species vs time data
+         - *lbls* [default=None] a list of species labels
+        """
+        self.species = species
+        self.HAS_SPECIES = True
+        if lbls != None:
+            self.species_labels = lbls   
+            
+    def setTime(self, time, lbl=None):
+        """
+        Set the time vector
+
+        Input:
+         - *time* a 1d array of time points
+         - *lbl* [default=None] is "Time" set as required
+        """
+        self.time = time.reshape(len(time), 1)
+        self.HAS_TIME = True
+        if lbl != None:
+            self.time_label = lbl
+
+    def getSpecies(self, lbls=False):
+        """
+        Return an array of time+species
+
+        Input:
+        - *lbls* [default=False] return only the time+species array or optionally both the data array and a list of column label
+        """
+        output = None
+        if self.HAS_SPECIES:
+            output = np.hstack((self.time, self.species))
+            labels = [self.time_label]+self.species_labels
+        else:
+            output = self.time
+            labels = [self.time_label]
+        if not lbls:
+            return output
+        else:
+            return output, labels   
+            
+    def getTime(self, lbls=False):
+        """
+        Return the time vector
+
+        Input:
+         - *lbls* [default=False] return only the time array or optionally both the time array and time label
+        """
+        output = None
+        if self.HAS_TIME:
+            output = self.time.reshape(len(self.time),)
+        if not lbls:
+            return output
+        else:
+            return output, [self.time_label]    
+            
+    def setPropensities(self, propensities):
+        """
+        Sets an array of propensities.
+
+        Input:
+         - *propensities* (list)
+        """ 
+        self.propensities = propensities
+        self.HAS_PROPENSITIES = True
+
+    def setSpeciesAutocorrelations(self,auto_correlations, lbls=None):
+        """
+        Set the `autocorrelations` ***
+
+        Input:
+         - *auto_correlation* a list of auto correlations
+         - *lbls* [default=None] a list of matching reaction names         
+        """
+        self.species_autocorrelations = auto_correlations
+        self.HAS_SPECIES_AUTOCORRELATIONS = True   
+
+    def setSpeciesAutocovariances(self,auto_covariances, lbls=None):
+        """
+        Set the `autocorrelations` ***
+
+
+        Input:
+         - *auto_correlation* a list of auto correlations
+         - *lbls* [default=None] a list of matching reaction names         
+        """
+        self.species_autocovariances = auto_covariances
+        self.HAS_SPECIES_AUTOCOVARIANCES = True       
+        
+    def setPropensitiesAutocorrelations(self,auto_correlations, lbls=None):
+        """
+        Set the `autocorrelations` ***
+
+        Input:
+         - *auto_correlation* a list of auto correlations
+         - *lbls* [default=None] a list of matching reaction names         
+        """
+        self.propensities_autocorrelations = auto_correlations
+        self.HAS_PROPENSITIES_AUTOCORRELATIONS = True
+
+    def setPropensitiesAutocovariances(self,auto_covariances, lbls=None):
+        """
+        Set the `autocovariances` ***
+
+
+        Input:
+         - *auto_correlation* a list of auto covariances
+         - *lbls* [default=None] a list of matching reaction names         
+        """
+        self.propensities_autocovariances = auto_covariances
+        self.HAS_PROPENSITIES_AUTOCOVARIANCES = True             
 
 class IntegrationStochasticDataObj(object):
     """
@@ -399,30 +756,67 @@ class IntegrationStochasticDataObj(object):
     time = None
     waiting_times = None
     species = None
+    species_distributions = None    
+    species_distributions_means = None
+    species_distributions_standard_deviations = None
+    
     propensities = None
+    propensities_distributions = None
+    propensities_distributions_means = None
+    propensities_distributions_standard_deviations = None
+    
     xdata = None
     time_label = 'Time'
-    waiting_time_labels = None
+    waiting_times_labels = None
     species_labels = None
     propensities_labels = None
     xdata_labels = None
     HAS_SPECIES = False
     HAS_WAITING_TIMES = False
-    HAS_PROPENSITIES = False
+    HAS_PROPENSITIES = False    
     HAS_TIME = False
-
     HAS_XDATA = False
     IS_VALID = True
     TYPE_INFO = 'Stochastic'
+    def setSpeciesDist(self,distributions,means,stds,moments):
+        """
+        setSpeciesDist stuff for the determination of distributions
+        
+        Input:
+         - *distributions* (list)
+         - *means* (dictionairy)
+         - *stds* (dictionairy)
+         - *moments* (dictionairy) 
+        """
+        self.species_distributions = distributions
+        self.species_means = means
+        self.species_standard_deviations = stds
+        self.species_moments = moments
 
-    def setDist(self,distributions,means,sds):
-        """ setDist stuff for the determination of distributions """
-        self.distributions = distributions
-        self.means = means
-        self.standard_deviations = sds
+    def setPropensitiesDist(self,distributions,means,stds,moments):
+        """
+        setPropensitiesDist stuff for the determination of distributions
+        
+        Input:
+         - *distributions* (list)
+         - *means* (dictionairy)
+         - *stds* (dictionairy)
+         - *moments* (dictionairy) 
+        """
+        self.propensities_distributions = distributions
+        self.propensities_means = means
+        self.propensities_standard_deviations = stds
+        self.propensities_moments = moments
 
     def setSimulationInfo(self,timesteps,endtime,simulation_trajectory):
-        """ setSimulationInfo"""
+        """
+        set Simulation Information
+        
+        Input:
+         - *timesteps* (integer)
+         - *endtime* (float)
+         - *simulation_trajectory* (integer)
+        """
         self.simulation_timesteps = timesteps
         self.simulation_endtime = endtime
         self.simulation_trajectory = simulation_trajectory
@@ -452,7 +846,6 @@ class IntegrationStochasticDataObj(object):
         Input:
          - *time* a 1d array of time points
          - *lbl* [default=None] is "Time" set as required
-
         """
         self.time = time.reshape(len(time), 1)
         self.HAS_TIME = True
@@ -472,18 +865,36 @@ class IntegrationStochasticDataObj(object):
         if lbls != None:
             self.species_labels = lbls
 
-    def setMeanWaitingtimes(self,waiting_times):
+    def setWaitingtimesMeans(self,waiting_times,rate_names):
         """
-        set Mean Waiting times
+        set waiting times means
         
         Input:
          - *waiting_times* (dictionary)
         """
-        self.mean_waitingtimes = []
-        for r in waiting_times:
-            waiting_times_r = waiting_times[r]
-            self.mean_waitingtimes.append(np.mean(waiting_times_r))
+        self.waiting_times_means = []
+        for r_id in rate_names:
+            waiting_times_r = waiting_times[r_id]
+            self.waiting_times_means.append(np.mean(waiting_times_r))
 
+    def setWaitingtimesStandardDeviations(self,waiting_times,rate_names):
+        """
+        set waiting times standard deviations
+        
+        Input:
+         - *waiting_times* (dictionary)
+        """
+        self.waiting_times_standard_deviations = []
+        i=0
+        for r_id in rate_names:
+            waiting_times_r = np.array(waiting_times[r_id])
+            if len(waiting_times_r):                
+                variance = sum((self.waiting_times_means[i]- waiting_times_r)**2)/len(waiting_times_r)
+                self.waiting_times_standard_deviations.append(variance**0.5)         
+            else:
+                self.waiting_times_standard_deviations.append(np.NAN)
+            i+=1
+          
 
     def setWaitingtimes(self, waiting_times, lbls=None):
         """
@@ -502,7 +913,7 @@ class IntegrationStochasticDataObj(object):
         self.waiting_times = waiting_times
         self.HAS_WAITING_TIMES = True
         if lbls != None:
-            self.waiting_time_labels = lbls
+            self.waiting_times_labels = lbls
 
     def setPropensitiesLabels(self,labels): 
         """
@@ -511,7 +922,7 @@ class IntegrationStochasticDataObj(object):
         """
         self.propensities_labels = labels            
 
-    def setPropensities(self, propensities):
+    def setPropensities(self, propensities,lbls=None):
         """
         Sets an array of propensities.
 
@@ -523,7 +934,9 @@ class IntegrationStochasticDataObj(object):
         for r in xrange(P_ARR.shape[0]):
             P_ARR[r, :] = propensities[r][1:]
         self.propensities = P_ARR
-        self.HAS_PROPENSITIES = True            
+        self.HAS_PROPENSITIES = True 
+        if lbls != None:
+            self.propensities_labels = lbls           
 
     def setXData(self, xdata, lbls=None):
         """
@@ -555,7 +968,7 @@ class IntegrationStochasticDataObj(object):
 
     def getSpecies(self, lbls=False):
         """
-        Return an array fo time+species
+        Return an array of time+species
 
         Input:
         - *lbls* [default=False] return only the time+species array or optionally both the data array and a list of column label
@@ -591,7 +1004,7 @@ class IntegrationStochasticDataObj(object):
             ##  else:
             for t in traj:
                 output.append(self.waiting_times[t])
-            labels = self.waiting_time_labels
+            labels = self.waiting_times_labels
         else:
             output = []
             labels = []
@@ -677,7 +1090,6 @@ class IntegrationStochasticDataObj(object):
                 output = np.hstack((output,ru))
             if xd is not None:
                 output = np.hstack((output,xd))
-
         return output
 
     def getDataInTimeInterval(self, time, bounds=None):
@@ -700,7 +1112,6 @@ class IntegrationStochasticDataObj(object):
         t = []
         sp = None
         ra = None
-
         for tt in xrange(len(c1)):
             if c1[tt] and c2[tt]:
                 t.append(tt)
