@@ -6,9 +6,9 @@ Example of a sequential simulator. This simulator tracks one cell for N generati
 
 Most of the functionalities of the SSA module such as plotting and writing to a file are also available in this module.
 
-Written by TR Maarleveld, Amsterdam, The Netherlands
+Written by T.R. Maarleveld, Amsterdam, The Netherlands
 E-mail: tmd200@users.sourceforge.net
-Last Change: September 21, 2012
+Last Change: April 05, 2013
 """
 
 ############################## IMPORTS ###################################
@@ -36,12 +36,14 @@ class CellDivision():
     >>> cmod.DoCellDivisionStochSim(self,distribution = 'gamma',mean = 10, sd = 3,generations = 5,IsTrackPropensities=False)
     >>> cmod.Model(File = 'filename.psc', dir = '/.../')
     >>> cmod.Reload()
-    >>> cmod.PlotTimeSim()
-    >>> cmod.PlotPropensities()  
-    >>> cmod.PlotWaitingtimes()
-    >>> cmod.PlotDistributions(bin_size = 3)
-    >>> cmod.ShowMeans()
-    >>> cmod.ShowStandardDeviations()
+    >>> smod.PlotSpeciesTimeSeries()
+    >>> smod.PlotPropensityTimeSeries()
+    >>> smod.PlotAveragedSpeciesTimeSeries()
+    >>> smod.PlotWaitingtimeDistributions()
+    >>> smod.PlotSpeciesDistributions(bin_size = 3)
+    >>> smod.PrintSpeciesMeans()
+    >>> smod.PrintSpeciesDeviations()
+    >>> smod.PrintPropensityMeans()
     >>> cmod.ShowOverview()  
     >>> cmod.ShowSpecies()
     """
@@ -60,7 +62,7 @@ class CellDivision():
         Method = 'Direct' # HARD CODED
         self.output_dir = output_dir
         self.model_dir = dir       
-        self.ssa_mod = StochSim.SSA(Method,File,dir,Mode = 'time',IsInteractive=True,IsTrackPropensities=False)         
+        self.ssa_mod = StochSim.SSA(Method,File,dir,Mode = 'time',IsTrackPropensities=False)         
         try: 
             Analysis.plt.ion()   # Set on interactive pylab environment
         except Exception,er:
@@ -123,7 +125,7 @@ class CellDivision():
             print "Info: Time simulation output of the trajectories is stored at %s in directory: %s" % (self.ssa_mod.model_file[:-4]+'(traj).dat',self.ssa_mod.temp_dir)
             print "Info: output is written to: %s" % os.path.join(self.ssa_mod.output_dir,'ssa_sim.log')
         t1 = time.time()
-        for self.ssa_mod.traj in xrange(1,self.ssa_mod.sim_trajectories+1):            
+        for self.ssa_mod.current_trajectory in xrange(1,self.ssa_mod.sim_trajectories+1):            
             if distribution.lower() == 'gamma':
                 self.cell_division_times = np.random.gamma(mean,sd,generations)
             elif distribution.lower() == 'lognormal':
@@ -153,7 +155,7 @@ class CellDivision():
             if self.ssa_mod.sim_trajectories == 1: 
                 print "Number of time steps %s End time %s" % (self.ssa_mod.SSA.timestep,self.ssa_mod.SSA.sim_t)
             elif self.ssa_mod.sim_trajectories > 1:
-                self.ssa_mod.DumpTrajectoryData(self.ssa_mod.traj)        
+                self.ssa_mod.DumpTrajectoryData(self.ssa_mod.current_trajectory)        
         t2 = time.time()  
         self.simulation_time = t2-t1
         print "Simulation time: %s" % self.simulation_time
@@ -206,7 +208,7 @@ class CellDivision():
         """ Print an overview of the current settings """
         self.ssa_mod.ShowOverview()
     
-    def PlotTimeSim(self,maxpoints2plot = 10000,species2plot = True,linestyle = 'solid',marker = '',colors = None,title = 'StochPy Time Simulation Plot'):
+    def PlotSpeciesTimeSeries(self,maxpoints2plot = 10000,species2plot = True,linestyle = 'solid',marker = '',colors = None,title = 'StochPy Species Time Course Plot'):
         """
         Plot time simulation output for each generated trajectory
         Default: PlotTimeSim() plots time simulation for each species
@@ -218,9 +220,9 @@ class CellDivision():
          - *marker* [default = ''] ('v','o','*',',')
          - *title* [default = 'StochPy Time Simulation Plot']  (string)
         """
-        self.ssa_mod.PlotTimeSim(maxpoints2plot,species2plot,linestyle,marker,colors,title)
+        self.ssa_mod.PlotSpeciesTimeSeries(maxpoints2plot,species2plot,linestyle,marker,colors,title)
 
-    def PlotDistributions(self,species2plot = True, linestyle = 'dotted',colors=None,title = 'StochPy Probability Density Function',bin_size=1):
+    def PlotSpeciesDistributions(self,species2plot = True, linestyle = 'dotted',colors=None,title = 'StochPy Species Probability Density Function',bin_size=1):
         """
         Plots the PDF for each generated trajectory
         Default: PlotDistributions() plots PDF for each species
@@ -229,12 +231,12 @@ class CellDivision():
          - *species2plot* [default = True] as a list ['S1','S2']
          - *linestyle* [default = 'dotted'] (string)
          - *colors* (list)
-         - *title* [default = 'StochPy Probability Density Function'] (string)     
+         - *title* [default = 'StochPy Species Probability Density Function'] (string)     
          - *bin_size* [default=1] (integer)
         """
-        self.ssa_mod.PlotDistributions(species2plot,linestyle,colors,title,bin_size)
+        self.ssa_mod.PlotSpeciesDistributions(species2plot,linestyle,colors,title,bin_size)
                 
-    def PlotPropensities(self,maxpoints2plot = 10000,rates2plot = True,linestyle = 'solid',marker = '',colors = None,title = 'StochPy Propensities Plot'):
+    def PlotPropensityTimeSeries(self,maxpoints2plot = 10000,rates2plot = True,linestyle = 'solid',marker = '',colors = None,title = 'StochPy Propensity Time Course Plot'):
         """ 
         Plot time simulation output for each generated trajectory
 
@@ -246,15 +248,29 @@ class CellDivision():
          - *marker* [default = ''] ('v','o','*',',')
          - *linestyle* [default = 'solid']: dashed, dotted, and solid (string)
          - *colors* [default = None] (list)
-         - *title* [default = 'StochPy Propensities Plot'] (string)
+         - *title* [default = 'StochPy Propensity Time Course Plot'] (string)
         """
-        self.ssa_mod.PlotPropensities(maxpoints2plot,rates2plot,linestyle,marker,colors,title)
+        self.ssa_mod.PlotPropensityTimeSeries(maxpoints2plot,rates2plot,linestyle,marker,colors,title)
+        
+    def PlotPropensityDistributions(self,rates2plot = True, linestyle = 'dotted',colors=None,title = 'StochPy Propensity Probability Density Function',bin_size=1):
+        """
+        Plots the PDF for each generated trajectory
+        Default: PlotDistributions() plots PDF for each species
+
+        Input:
+         - *rates2plot* [default = True] as a list ['R1','R2']
+         - *linestyle* [default = 'dotted'] (string)
+         - *colors* (list)
+         - *title* [default = 'StochPy Propensity Probability Density Function'] (string)     
+         - *bin_size* [default=1] (integer)
+        """
+        self.ssa_mod.PlotPropensityDistributions(rates2plot,linestyle,colors,title,bin_size)   
 
     def GetWaitingtimes(self):
         """ Get for each reaction the waiting times """ 
         self.ssa_mod.GetWaitingtimes()
 
-    def PlotWaitingtimes(self,rates2plot = True,linestyle = 'None',marker = 'o',colors = None,title = 'StochPy Waitingtimes Plot'):
+    def PlotWaitingtimeDistributions(self,rates2plot = True,linestyle = 'None',marker = 'o',colors = None,title = 'StochPy Waitingtimes Plot'):
         """
         Plot obtained waiting times
         default: PlotWaitingtimes() plots waiting times for all rates
@@ -266,18 +282,18 @@ class CellDivision():
          - *colors* [default =  None] (list)
          - *title* [default = 'StochPy Waitingtimes Plot'] (string)
         """    
-        self.ssa_mod.PlotWaitingtimes(rates2plot,linestyle,marker,colors,title)
+        self.ssa_mod.PlotWaitingtimeDistributions(rates2plot,linestyle,marker,colors,title)
 
-    def GetInterpolatedData(self,frames=51):
+    def GetRegularGrid(self,npoints=51):
         """
         Perform linear interpolation for each generated trajectory. Linear interpolation is done for all integer time points, between the start time (0) end the endtime.
     
         Input:
          - *frames* (int)
         """    
-        self.ssa_mod.GetInterpolatedData(frames)
+        self.ssa_mod.GetRegularGrid(npoints)  
         
-    def PlotInterpolatedData(self,species2plot = True,linestyle = 'None',marker = 'o',colors = None,title = 'StochPy Interpolated Time Plot (# of trajectories = )'): 
+    def PlotPlotAveragedSpeciesTimeSeries(self,species2plot = True,linestyle = 'None',marker = 'o',colors = None,title = 'StochPy Interpolated Time Plot (# of trajectories = )'): 
         """
         Plot the averaged interpolation result. For each time point, the mean and standard deviation are plotted 
         
@@ -288,22 +304,32 @@ class CellDivision():
          - *colors* [default =  None] (list)
          - *title* [default = StochPy Interpolated Time (# of trajectories = ... ) ] (string)
         """      
-        self.ssa_mod.PlotInterpolatedData(species2plot,linestyle,marker,colors,title)    
+        self.ssa_mod.PlotAveragedSpeciesTimeSeries(species2plot,linestyle,marker,colors,title)    
               
-    def ShowMeans(self):
+    def PrintSpeciesMeans(self):
         """ Print the means of each species for the selected trajectory"""
-        self.ssa_mod.ShowMeans()
+        self.ssa_mod.PrintSpeciesMeans()
 
-    def ShowStandardDeviations(self):
+    def PrintSpeciesDeviations(self):
         """ Print the standard deviations of each species for the selected trajectory"""      
-        self.ssa_mod.ShowStandardDeviations()
+        self.ssa_mod.PrintSpeciesStandardDeviations()
+        
+    def PrintPropensityMeans(self):
+        """ Print the means of each species for the selected trajectory"""
+        self.ssa_mod.PrintPropensityMeans()
 
-    def Export2File(self,what='TimeSim', directory=None):
+    def PrintSpeciesDeviations(self):
+        """ Print the standard deviations of each species for the selected trajectory"""      
+        self.ssa_mod.PrintPropensityStandardDeviations()
+
+    def Export2File(self,analysis='timeseries',datatype='species', IsAverage = False, directory=None):
         """
         Export output to a file
 
-        Input:
-         - *what* [default = TimeSim] TimeSim, Propensities, Distributions, Waitingtimes, and Interpol (string)
-         - *directory* [default = None] (string)
+      Input:
+       - *analysis* [default = 'timeseries'] (string) options: timeseries, distribution, mean, std, autocorrelation, autocovariance
+       - *datatype*  [default = 'species'] (string) options: species, propensities, waitingtimes
+       - *IsAverage* [default = False] (boolean)   
+       - *directory* [default = None] (string)
         """
-        self.ssa_mod.Export2File(what,directory)
+        self.ssa_mod.Export2File(analysis,datatype,IsAverage,directory)
